@@ -44,16 +44,16 @@ class MqttConnector(Connector, Thread):
         self.__broker = config.get('broker')
         self.__hostIP = hostIP
         log.info("MQTT CONNECTOR: %s ", hostIP)
-
+        
         self.__mapping = []
         self.__server_side_rpc = []
         self.__connect_requests = []
         self.__disconnect_requests = []
         self.__attribute_requests = []
         self.__attribute_updates = []
-        # self.lastDataDict = {}
-        # self.lastDataDict[clientID] = {}
-        #self.initLastDataDict(clientID)
+        self.lastDataDict = {}
+        self.lastDataDict[clientID] = {}
+        self.initLastDataDict(clientID)
         mandatory_keys = {
             "mapping": ['topicFilter', 'converter'],
             # "serverSideRpc": ['deviceNameFilter', 'methodFilter', 'requestTopicExpression', 'valueExpression'],
@@ -67,19 +67,19 @@ class MqttConnector(Connector, Thread):
         self.load_handlers('mapping', mandatory_keys['mapping'], self.__mapping)
 
         # RPCs, i.e., remote procedure calls (ThingsBoard towards devices) handlers
-        # self.load_handlers('serverSideRpc', mandatory_keys['serverSideRpc'], self.__server_side_rpc)
+        #self.load_handlers('serverSideRpc', mandatory_keys['serverSideRpc'], self.__server_side_rpc)
 
         # Connect requests, i.e., telling ThingsBoard that a device is online even if it does not post telemetry
-        # self.load_handlers('connectRequests', mandatory_keys['connectRequests'], self.__connect_requests)
+        #self.load_handlers('connectRequests', mandatory_keys['connectRequests'], self.__connect_requests)
 
         # Disconnect requests, i.e., telling ThingsBoard that a device is offline even if keep-alive has not expired yet
-        # self.load_handlers('disconnectRequests', mandatory_keys['disconnectRequests'], self.__disconnect_requests)
+        #self.load_handlers('disconnectRequests', mandatory_keys['disconnectRequests'], self.__disconnect_requests)
 
         # Shared attributes direct requests, i.e., asking ThingsBoard for some shared attribute value
-        # self.load_handlers('attributeRequests', mandatory_keys['attributeRequests'], self.__attribute_requests)
+        #self.load_handlers('attributeRequests', mandatory_keys['attributeRequests'], self.__attribute_requests)
 
         # Attributes updates requests, i.e., asking ThingsBoard to send updates about an attribute
-        # self.load_handlers('attributeUpdates', mandatory_keys['attributeUpdates'], self.__attribute_updates)
+        #self.load_handlers('attributeUpdates', mandatory_keys['attributeUpdates'], self.__attribute_updates)
 
         # Setup topic substitution lists for each class of handlers ----------------------------------------------------
         self.__mapping_sub_topics = {}
@@ -88,10 +88,10 @@ class MqttConnector(Connector, Thread):
         self.__attribute_requests_sub_topics = {}
 
         # Set up external MQTT broker connection -----------------------------------------------------------------------
-        self.__client_ID = clientID  # self.__broker.get("clientId", ''.join(random.choice(string.ascii_lowercase) for _ in range(23)))
+        self.__client_ID = clientID #self.__broker.get("clientId", ''.join(random.choice(string.ascii_lowercase) for _ in range(23)))
         self._client = Client(self.__client_ID)
         self.setName(self.__client_ID + " " + hostIP)
-        # config.get("name", self.__broker.get("name", 'Mqtt Broker ' + ''.join(random.choice(string.ascii_lowercase) for _ in range(5)))))
+        #config.get("name", self.__broker.get("name", 'Mqtt Broker ' + ''.join(random.choice(string.ascii_lowercase) for _ in range(5)))))
 
         self._client.username_pw_set(user, password)
 
@@ -181,8 +181,6 @@ class MqttConnector(Connector, Thread):
                 if not self._connected:
                     sleep(1)
             except ConnectionRefusedError as e:
-                self.__log.error('ERROR ON IP: ')
-                self.__log.error(self.__hostIP)
                 self.__log.error(e)
                 sleep(10)
 
@@ -251,10 +249,9 @@ class MqttConnector(Connector, Thread):
                             "Cannot find converter for %s topic", mapping["topicFilter"])
                         continue
 
-                    topic = self.remove_nodeid(mapping["topicFilter"])
                     # Setup regexp topic acceptance list ---------------------------------------------------------------
                     regex_topic = TBUtility.topic_to_regex(
-                        self.check_topic(topic))
+                        mapping["topicFilter"])
 
                     # There may be more than one converter per topic, so I'm using vectors
                     if not self.__mapping_sub_topics.get(regex_topic):
@@ -370,57 +367,36 @@ class MqttConnector(Connector, Thread):
     def initLastDataDict(self, clientID):
         for element in self.config['mapping']:
             self.__log.info("FILTER: %s", element['topicFilter'])
-            topic = self.remove_nodeid(element['topicFilter'])
-            self.lastDataDict[clientID][self.check_topic(topic)] = {"ts": time(), "value": 1}
+            self.lastDataDict[clientID][element['topicFilter']] = {"ts": time(), "value": 1}
             self.__log.info(self.lastDataDict)
-            # {'RTU003': {'/v2/rt.di.reg5': 1}}
+            #{'RTU003': {'/v2/rt.di.reg5': 1}}
 
     def checkTelemetryUpdate(self, clientID, topic, content):
         self.__log.info(self.lastDataDict)
         if self.lastDataDict[clientID][topic]['value'] == content:
-            # return False if more than 1 minute has passed since last update
-            if time() - self.lastDataDict[clientID][topic]['ts'] > self.config.get("update_interval_seconds", 60):
-                self.__log.info("Update: %s", content)
-                self.lastDataDict[clientID][topic]['ts'] = time()
-                self.lastDataDict[clientID][topic]['value'] = content
-                return False
-            else:
-                self.__log.info("Same")
-                return True
+                #return False if more than 1 minute has passed since last update
+                if time() - self.lastDataDict[clientID][topic]['ts'] > self.config.get("update_interval_seconds", 60):
+                    self.__log.info("Update: %s", content)
+                    self.lastDataDict[clientID][topic]['ts'] = time()
+                    self.lastDataDict[clientID][topic]['value'] = content
+                    return False
+                else:
+                    self.__log.info("Same")
+                    return True
         else:
             self.lastDataDict[clientID][topic]['value'] = content
             self.__log.info(self.lastDataDict)
             return False
 
-    def check_topic(self, topic):
-        import re
-        if re.search('\[.*?\]', topic):
-            len = topic.count('[')
-            for i in range(len):
-                topic = topic.replace('[', 'X')
-                topic = topic.replace(']', '')
-        return topic
-
-    def remove_nodeid(self,  topic):
-        node_id = topic.split("/")[1]
-        return topic.replace('/' + node_id, '')
-
     def _on_message(self, client, userdata, message):
         self.statistics['MessagesReceived'] += 1
-        topic = message.topic
-        node_id = message.topic.split("/")[1]
-        # if node_id != self.__client_ID.split('-')[1]:
-        #     return
-        # else:
-        topic = topic.replace('/' + node_id, '')
-        valid_topic = self.check_topic(topic)
         content = TBUtility.decode(message)
-        # if self.is_connected():
-            # if self.checkTelemetryUpdate(self.__client_ID, valid_topic, content) is True:
-            #     return
+        if self.is_connected():
+            if self.checkTelemetryUpdate(self.__client_ID, message.topic, content) is True:
+                return
         # Check if message topic exists in mappings "i.e., I'm posting telemetry/attributes" ---------------------------
         topic_handlers = [
-            regex for regex in self.__mapping_sub_topics if fullmatch(regex, valid_topic)]
+            regex for regex in self.__mapping_sub_topics if fullmatch(regex, message.topic)]
 
         if topic_handlers:
             # Note: every topic may be associated to one or more converter. This means that a single MQTT message
@@ -567,7 +543,7 @@ class MqttConnector(Connector, Thread):
                     handler = self.__attribute_requests_sub_topics[topic]
 
                     found_device_name = None
-                    found_attribute_names = None
+                    found_attribute_name = None
 
                     # Get device name, either from topic or from content
                     if handler.get("deviceNameTopicExpression"):
@@ -584,28 +560,30 @@ class MqttConnector(Connector, Thread):
                         attribute_name_match = search(
                             handler["attributeNameTopicExpression"], message.topic)
                         if attribute_name_match is not None:
-                            found_attribute_names = attribute_name_match.group(0)
+                            found_attribute_name = attribute_name_match.group(
+                                0)
                     elif handler.get("attributeNameJsonExpression"):
-                        found_attribute_names = list(filter(lambda x: x is not None,
-                                                            TBUtility.get_values(handler["attributeNameJsonExpression"],
-                                                                                 content)))
+                        found_attribute_name = TBUtility.get_value(
+                            handler["attributeNameJsonExpression"], content)
 
                     if found_device_name is None:
                         self.__log.error(
                             "Device name missing from attribute request")
                         continue
 
-                    if found_attribute_names is None:
-                        self.__log.error("Attribute name missing from attribute request")
+                    if found_attribute_name is None:
+                        self.__log.error(
+                            "Attribute name missing from attribute request")
                         continue
 
-                    self.__log.info("Will retrieve attribute %s of %s", found_attribute_names, found_device_name)
+                    self.__log.info("Will retrieve attribute %s of %s",
+                                    found_attribute_name, found_device_name)
                     self.__gateway.tb_client.client.gw_request_shared_attributes(
                         found_device_name,
-                        found_attribute_names,
+                        [found_attribute_name],
                         lambda data, *args: self.notify_attribute(
                             data,
-                            found_attribute_names,
+                            found_attribute_name,
                             handler.get("topicExpression"),
                             handler.get("valueExpression"),
                             handler.get('retain', False)))
@@ -632,21 +610,18 @@ class MqttConnector(Connector, Thread):
                          content)
 
     def notify_attribute(self, incoming_data, attribute_name, topic_expression, value_expression, retain):
-        if incoming_data.get("device") is None or incoming_data.get("value", incoming_data.get('values')) is None:
+        if incoming_data.get("device") is None or incoming_data.get("value") is None:
             return
 
         device_name = incoming_data.get("device")
-        attribute_values = incoming_data.get('value') or incoming_data.get('values')
+        attribute_value = incoming_data.get("value")
 
         topic = topic_expression \
             .replace("${deviceName}", str(device_name)) \
             .replace("${attributeKey}", str(attribute_name))
 
-        if len(attribute_name) <= 1:
-            data = value_expression.replace("${attributeKey}", str(attribute_name[0])) \
-                .replace("${attributeValue}", str(attribute_values))
-        else:
-            data = simplejson.dumps(attribute_values)
+        data = value_expression.replace("${attributeKey}", str(attribute_name)) \
+            .replace("${attributeValue}", str(attribute_value))
 
         self._client.publish(topic, data, retain=retain).wait_for_publish()
 
@@ -759,7 +734,8 @@ class MqttConnector(Connector, Thread):
 
                 data_to_send = rpc_config.get('valueExpression')
                 for (tag, value) in zip(data_to_send_tags, data_to_send_values):
-                    data_to_send = data_to_send.replace('${' + tag + '}', simplejson.dumps(value))
+                    data_to_send = data_to_send.replace(
+                        '${' + tag + '}', str(value))
 
                 try:
                     self.__log.info("Publishing to: %s with data %s",
